/*
 * normal_query.cpp
 *
 *  Created on: 2011/04/13
 *      Author: shu
 */

#include "aligner_parameters.h"
#include "encoder.h"
#include "sequence.h"
#include "statistics.h"
#include "sa_sequence_index.h"
//#include "fmi_sequence_index.h"
#include "query.h"
#include "protein_query.h"
#include <string>
#include <stdexcept>
#include <vector>
#include <iostream>
#include "../ext/seg/src/seg.h"

using namespace std;

ProteinQuery::ProteinQuery(const Sequence &sequence,
		AlignerParameters &parameters) :
		Query(parameters), parameters_(&parameters), encoder_(
				*(parameters.GetAlignedType())), is_set_statical_parameters_(
				false) {
	SetName(sequence);
	SetEncodedSequence(sequence, encoder_, parameters.GetFilterFlag());
	//SetIndex(sequence_, encoder_);
	//SetStaticalParameters(sequence_, parameters);
}

void ProteinQuery::SetName(const Sequence &sequence) {
	name_ = sequence.GetName();
}

void ProteinQuery::SetEncodedSequence(const Sequence &sequence,
		const Encoder &encoder, const bool filter) {

	seg::Seg seg(kSegWindow);
	sequence_.resize(sequence.GetSequenceData().length() + 2);
	sequence_[0] = AlignerParameters::kSequenceEnd;
	if (!filter || sequence.GetSequenceData().length() < kSegWindow) {
		for (uint32_t i = 0; i < sequence.GetSequenceData().length(); ++i) {
			sequence_[1 + i] = encoder.GetUnknownCode();
		}
	} else {
		string masked_sequence;
		seg.MaskSequence(sequence.GetSequenceData(), &masked_sequence);
		encoder.Encode(masked_sequence, &sequence_[1]);
	}
	sequence_[sequence_.size() - 1] = AlignerParameters::kSequenceEnd;
}

void ProteinQuery::SetIndex(const vector<Code> &sequence,
		const Encoder &encoder) {
	index_ = new SaSequenceIndex(&sequence[0], sequence.size(), encoder);
}
void ProteinQuery::SetStaticalParameters(const vector<Code> &sequence,
		const AlignerParameters &parameters) {
	Statistics statistics(*(parameters.GetAlignedType()));
	try {
		statistics.CalculateUngappedKarlinParameters(&sequence[0],
				sequence.size(), parameters.GetScoreMatrix(),
				&ungapped_karlin_parameters_);
	} catch (invalid_argument e) {
		ungapped_karlin_parameters_ =
				parameters.GetUnGappedIdealKarlinParameters();
	}
	ungapped_extend_nominal_cutoff_ = Statistics::Normalized2Nominal(
			parameters.GetUngappedExtendNormalizedCutoff(),
			ungapped_karlin_parameters_);
	if (ungapped_extend_nominal_cutoff_
			> parameters.GetUngappedIdealExtendNominalCutoff()) {
		ungapped_extend_nominal_cutoff_ =
				parameters.GetUngappedIdealExtendNominalCutoff();
	}
	gapped_extend_nominal_cutoff_ =
			parameters.GetGappedIdealExtendNominalCutoff();
	gapped_extend_nominal_trigger_ = Statistics::Normalized2Nominal(
			parameters.GetGappedExtendNormalizedTrigger(),
			ungapped_karlin_parameters_);
	if (gapped_extend_nominal_trigger_
			> parameters.GetGappedIdealExtendNominalTrigger()) {
		gapped_extend_nominal_trigger_ =
				parameters.GetGappedIdealExtendNominalTrigger();
	}
	is_set_statical_parameters_ = true;
}

